%  Equations for VDP (x'' + alpha x' - lambda(1-x^2)x' + x = 0) :
%
% Order 1 recast :
%  x' = y
%  y' = -alpha y + lambda (1 - x^2)y - x
%
% Two auxiliary variables are added :
%   v = lambda y
%   r = 1-x^2
%
%  The equilibrium is given by :
%  0 = y
%  0 = -alpha y + v*r - x 
%
% This system is known to encounter a Hopf bifurcation for lambda = alpha.

global U Section Diagram    % Global variables to export point from the diagram. 

addpath('../../SRC')

%% parameters of the system
neq = 2;
neq_aux = 2;

% specific parameters
parameters.alpha = 0.5;

%% initialization of the system
sys=SystAQ(neq,neq_aux,@equations,@point_display,@global_display,parameters);
 
%% starting point
x = 0;
y = 0;
lambda = 0;

%%% Auxiliary variables
v = lambda*y;
r = 1-x^2;

U0 = [x y lambda v r]';

%% Launch Manlab
Manlab('sys'             ,sys, ...      
       'U0value'         ,U0, ...
       'NRthreshold'     ,1e-10, ...
       'ANMthreshold'    ,1e-14, ...
       'Amax_max'        ,1, ...     % Maximum value of the domain of validity of the series.
       'StabilityCheck'  ,1, ...
       'displayvariables',[3 1; 3 2]);     % MANLAB run


